@@ -782,11 +782,11 @@ unrolled:
               // text insertions are allowed
               while (err_ < max_)
               {
-                ++err_;
-                ch = get();
                 // reached the end?
+                ch = get();
                 if (ch == EOF)
                   break;
+                ++err_;
                 // skip one (multibyte) char
                 if (!bin_ && ch >= 0xC0)
                 {
@@ -1054,7 +1054,7 @@ unrolled:
         if (!at_end())
         {
           // fuzzy search with find() can safely advance on a single prefix char of the regex
-          if (pos_ > cur_)
+          if (cur_ < pos_)
           {
             // this part is based on advance() in matcher.cpp, limited to advancing ahead till the one of the first pattern char(s) match
             size_t loc = cur_ + 1;
@@ -1075,9 +1075,9 @@ unrolled:
                     goto scan;
                   }
                   loc = e - buf_;
-                  set_current_and_peek_more(loc - 1);
-                  loc = cur_ + 1;
-                  if (loc >= end_)
+                  set_current_and_peek_more(loc);
+                  loc = cur_;
+                  if (loc >= end_ && eof_)
                     break;
                 }
               }
@@ -1096,9 +1096,9 @@ unrolled:
                   goto scan;
                 }
                 loc = e - buf_;
-                set_current_and_peek_more(loc - 1);
-                loc = cur_ + 1;
-                if (loc + pat_->len_ > end_)
+                set_current_and_peek_more(loc);
+                loc = cur_;
+                if (loc + pat_->len_ > end_ && eof_)
                   break;
               }
             }
