@@ -60,6 +60,10 @@ sequence of characters:
   a `char*` string, a `wchar_t*` wide string, a `std::string`, a
   `std::wstring`, a `FILE*` descriptor, or a `std::istream` object.
 
+- Strings specified as input must be persistent and cannot be temporary.  The
+  input string contents are incrementally extracted and converted as necessary,
+  when specified with an encoding format `reflex::Input::file_encoding`.
+
 - When assigned a wide string source as input, the wide character content is
   automatically converted to an UTF-8 character sequence when reading with
   get().  Wide strings are UCS-2/UCS-4 and may contain UTF-16 surrogate pairs.
@@ -313,11 +317,16 @@ class Input {
     static const file_encoding_type koi8_r     = 35; ///< KOI8-R
     static const file_encoding_type koi8_u     = 36; ///< KOI8-U
     static const file_encoding_type koi8_ru    = 37; ///< KOI8-RU
-    static const file_encoding_type custom     = 38; ///< custom code page
+    static const file_encoding_type null_data  = 38; ///< swap NUL with LF
+    static const file_encoding_type custom     = 39; ///< custom code page
   };
-  /// FILE* handler functor base class to handle FILE* errors and non-blocking FILE* reads
+  /// FILE* handler functor base class to analyze FILE* input, handle errors and non-blocking FILE* reads
   struct Handler {
-    virtual int operator()(FILE*) = 0;
+    virtual size_t operator()(
+        FILE*, ///< open file
+        char*, ///< pointer to buffer with data read from file, may be changed by handler
+        size_t ///< length of the buffered data, should be returned by the functor, or shorter to ignore or zero to force EOF and stop reading
+        ) = 0;
     virtual ~Handler() { };
   };
   /// Stream buffer for reflex::Input, derived from std::streambuf.
@@ -709,8 +718,17 @@ class Input {
       while (true)
       {
         size_t k = file_get(s, n);
-        if (k > 0 || feof(file_) || handler_ == NULL || (*handler_)(file_) == 0)
+        // invoke handler when set, pass FILE* and buffer pointer and size, handler returns equal or lesser size
+        if (handler_ != NULL)
+          k = (*handler_)(file_, s, k);
+        if (k > 0)
           return k;
+        // if a non-recoverable error occurred, then return 0, permit non-blocking IO errors to continue
+        if (errno != EAGAIN && errno != EWOULDBLOCK && errno != EINTR)
+          return 0;
+        // wait until non-blocking IO is ready for reading, or EOF
+        if (!file_ready())
+          return 0;
       }
     }
     if (istream_)
@@ -753,11 +771,31 @@ class Input {
   void file_size();
   /// Called by size() for a std::istream.
   void istream_size();
-  /// Implements get() on a FILE*.
+  /// Implements get() on a FILE*, is non-blocking if file is non-blocking, get() blocks to read at least one byte.
   size_t file_get(
       char  *s, ///< points to the string buffer to fill with input
       size_t n) ///< size of buffer pointed to by s
       ;
+  /// Read n bytes into buffer s from file, block when IO is non-blocking, return false when fewer bytes read on failure or eof.
+  bool file_read(
+      char  *s, ///< points to the string buffer to file with input
+      size_t n) ///< size of the buffer pointed to by s
+  {
+    if (file_ == NULL)
+      return false;
+    while (true)
+    {
+      size_t k = ::fread(s, 1, n, file_);
+      s += k;
+      n -= k;
+      if (n == 0)
+        return true;
+      if (!file_ready())
+        return false;
+    }
+  }
+  /// Wait for non-blocking file input to be ready and return true or return false for eof and errors, ignore EINTR
+  bool file_ready();
   /// Set FILE* handler
   void set_handler(Handler *handler)
   {
